//  FILE:       Execute.cpp
//              Microsoft TPC-H Kit Ver. 1.00
//              Copyright Microsoft, 1999
//              All Rights Reserved
//
//
//  PURPOSE:    Implementation of CExecute.
//  Contact:    Reshma Tharamal (reshmat@microsoft.com)
// 
#include "stdafx.h"

#include "ExecuteDll.h"
#include "SMExecute.h"
#include "Execute.h"

extern SQLHENV henv;

extern SM_Connection_Info	*p_Connections;				// Pointer to open connections
extern int					iConnectionCount;			// Number of open connections
extern CRITICAL_SECTION		hConnections;				// Critical section to serialize access to available connections

#ifdef _TPCH_AUDIT
	extern FILE *pfLogFile;									// Log file containing timestamps
	extern CRITICAL_SECTION hLogFileWrite;					// Handle to critical section
#endif

/////////////////////////////////////////////////////////////////////////////
// CExecute

char * g_szOdbcOps[] = { 
		"SQLAllocHandle",
		"SQLDriverConnect",
		"SQLExecDirect",
		"SQLSetStmtAttr",
		"SQLCancel",
		"SQLNumResultCols",
		"SQLDescribeCol",
		"SQLColAttribute",
		"SQLFetch",
		"SQLGetData",
		"SQLRowCount",
		"SQLMoreResults",
		"SQLBindCol"
	};

char * CExecError::m_szExecErrorDesc[] = { 
		"Connection is already in use."
	};

STDMETHODIMP CExecute::InterfaceSupportsErrorInfo(REFIID riid)
{
	static const IID* arr[] = 
	{
		&IID_IExecute
	};
	for (int i=0; i < sizeof(arr) / sizeof(arr[0]); i++)
	{
		if (InlineIsEqualGUID(*arr[i],riid))
			return S_OK;
	}
	return S_FALSE;
}

STDMETHODIMP CExecute::put_OutputFile(BSTR newVal)
{
	assert(m_pOutputFile);
	m_OutputFile = newVal;

	HRESULT	hr = m_pOutputFile->put_FileName(newVal);
	if FAILED(hr) 
	{
		m_pOutputFile->Release();
		m_pOutputFile = NULL;
	}
	return hr;
}

//DEL STDMETHODIMP CExecute::put_LogFile(BSTR newVal)
//DEL {
//DEL 	assert(m_pLogFile);
//DEL 
//DEL 	m_pLogFile->put_FileName(newVal);
//DEL 	return S_OK;
//DEL }

STDMETHODIMP CExecute::put_ErrorFile(BSTR newVal)
{
	assert(m_pErrorFile);
	m_ErrorFile = newVal;

	HRESULT	hr = m_pErrorFile->put_FileName(newVal);
	if FAILED(hr) 
	{
		m_pErrorFile->Release();
		m_pErrorFile = NULL;
	}
	return hr;
}

STDMETHODIMP CExecute::DoExecute(BSTR szCommand, BSTR szExecutionDtls, ExecutionType ExecMethod, \
								 BOOL bNoCount, BOOL bNoExecute, BOOL bParseOnly, BOOL bQuotedIds, \
								 BOOL bAnsiNulls, BOOL bShowQP, BOOL bStatsTime, BOOL bStatsIO, \
								 long lRowCount, long lQueryTmout, BSTR szConnection)
{
    HANDLE		hThrd;         
    DWORD		tid;         

   _CrtSetReportFile(_CRT_WARN, _CRTDBG_FILE_STDOUT);

   	m_szCommand = szCommand;
	m_szExecDtls = szExecutionDtls;

	m_ExecMthd = ExecMethod;
	if (m_ExecMthd == execODBC)
    {
        m_bNoCount = bNoCount;
		m_bNoExecute = bNoExecute;
		m_bParseOnly = bParseOnly;
		m_bQuotedIds = bQuotedIds;
		m_bAnsiNulls = bAnsiNulls;
		m_bShowQP = bShowQP;
		m_bStatsTime = bStatsTime;
		m_bStatsIO = bStatsIO;
		m_lRowCount = lRowCount;
		m_lQueryTmout = lQueryTmout;
		m_szConnection = szConnection;
    }

	if((hThrd = CreateThread( 0, 0, (LPTHREAD_START_ROUTINE)ExecutionThread,
			this, 0, &tid)) == NULL)         
		return(RaiseSystemError());
	
	CloseHandle(hThrd);

	return S_OK;
}

STDMETHODIMP CExecute::Abort()
{
	if (m_ExecMthd == execShell)
		return(AbortShell());
	else
		return(AbortODBC());
}

void ExecutionThread(LPVOID lpParameter)
{

	CExecute	*MyExecute = (CExecute*)lpParameter;

	MyExecute->m_tElapsedTime = 0;

	GetLocalTime(&MyExecute->m_tStartTime);
	MyExecute->PostMessage(WM_TASK_START, 0, 0);

#ifdef _TPCH_AUDIT
	char		szBuffer[MAXLOGCMDBUF];
	char		szFmt[MAXBUFLEN];

	sprintf(szFmt, "Start Step: '%%.%ds' at '%d/%d/%d %d:%d:%d:%d'\n", 
		MAXLOGCMDLEN,
		MyExecute->m_tStartTime.wMonth, MyExecute->m_tStartTime.wDay, 
		MyExecute->m_tStartTime.wYear, MyExecute->m_tStartTime.wHour, 
		MyExecute->m_tStartTime.wMinute, MyExecute->m_tStartTime.wSecond, 
		MyExecute->m_tStartTime.wMilliseconds);
	if (MyExecute->m_ExecMthd == execShell)
		WriteFileToTpchLog((LPSTR)MyExecute->m_szCommand, szFmt);
	else
	{
		sprintf(szBuffer, szFmt, (LPSTR)MyExecute->m_szCommand);
		WriteToTpchLog(szBuffer);
	}
#endif

	// Initialize the run status for the step to running. The completion status for
	// the step will be initialized by the Shell and ODBC execution functions.
    MyExecute->m_StepStatus = gintRunning;

	if (MyExecute->m_ExecMthd == execShell)
		MyExecute->m_tElapsedTime = MyExecute->ExecuteShell();
	else
		MyExecute->m_tElapsedTime = MyExecute->ExecuteODBC();

	// Close the output, log and error files
	if (MyExecute->m_pOutputFile)
		MyExecute->m_pOutputFile->Release();
	MyExecute->m_pOutputFile = NULL;
	
	MyExecute->m_ExecTime = NULL;

	GetLocalTime(&MyExecute->m_tEndTime);
	
#ifdef _TPCH_AUDIT
	sprintf(szFmt, "Complete Step: '%%.%ds' at '%d/%d/%d %d:%d:%d:%d'\n", 
		MAXLOGCMDLEN,
		MyExecute->m_tEndTime.wMonth, MyExecute->m_tEndTime.wDay, 
		MyExecute->m_tEndTime.wYear, MyExecute->m_tEndTime.wHour, 
		MyExecute->m_tEndTime.wMinute, MyExecute->m_tEndTime.wSecond, 
		MyExecute->m_tEndTime.wMilliseconds);
	if (MyExecute->m_ExecMthd == execShell)
		WriteFileToTpchLog((LPSTR)MyExecute->m_szCommand, szFmt);
	else
	{
		sprintf(szBuffer, szFmt, (LPSTR)MyExecute->m_szCommand);
		WriteToTpchLog(szBuffer);
	}
#endif

	MyExecute->PostMessage(WM_TASK_FINISH, 0, 0);

	return;
}

#ifdef _TPCH_AUDIT

void WriteFileToTpchLog(LPSTR szFile, LPSTR szFmt)
{
	// Reads a maximum of MAXLOGCMDBUF characters from the command file and writes it to the log
	FILE	*fpCmd;
	int		iRead;
	char	szBuf[MAXLOGCMDBUF];
	char	szCmd[MAXLOGCMDLEN];

	if ( pfLogFile != NULL )
	{
		if ( (fpCmd = fopen(szFile, FILE_ACCESS_READ)) != NULL)
		{
			iRead = fread(szCmd, sizeof(char), sizeof(szCmd) / sizeof(char), fpCmd);
			if (iRead < MAXLOGCMDLEN)
				szCmd[iRead] = '\0';
			else
				szCmd[MAXLOGCMDLEN - 1] = '\0';
			sprintf(szBuf, szFmt, szCmd);
			WriteToTpchLog(szBuf);
			fclose(fpCmd);
		}
	}
}

void WriteToTpchLog(char *szMsg)
{
	if (pfLogFile != NULL)
	{
		EnterCriticalSection(&hLogFileWrite);
		fprintf(pfLogFile, szMsg);
		LeaveCriticalSection(&hLogFileWrite);
	}
	
	return;
}
#endif

TC_TIME CExecute::ExecuteShell()
{
	STARTUPINFOA			Start;
	PROCESS_INFORMATION		proc;
	DWORD					exitCode;
	TC_TIME					tElapsed = 0;
	_bstr_t					szCommand("cmd /c ");
	LPSTR					szStartDir;
	CURRENCY				Elapsed;

	szCommand += m_szCommand;

	// Redirect output and error information 
	szCommand += " > " + m_OutputFile + " 2> " + m_ErrorFile;

	// Initialize the STARTUPINFO structure:
	memset(&Start, 0, sizeof(STARTUPINFOA)) ;
	Start.cb              = sizeof(Start);
	Start.dwFlags         = STARTF_USESHOWWINDOW;
	Start.wShowWindow     = SW_SHOWMINNOACTIVE;
	
	memset(&proc, 0, sizeof(PROCESS_INFORMATION)) ;

	szStartDir = strcmp((LPCTSTR)m_szExecDtls, "") == 0 ? NULL : (LPSTR)m_szExecDtls;

	m_ExecTime->Start();

	// Start the shelled application:
	if (!CreateProcessA( NULL, (LPSTR)szCommand, NULL, NULL, FALSE,
		NORMAL_PRIORITY_CLASS, NULL, szStartDir, &Start, &proc ))
	{
		m_StepStatus = gintFailed;
		LogSystemError(m_pErrorFile);
		
		m_ExecTime->Stop(&Elapsed);
		return((TC_TIME)Elapsed.int64);
	}
	
	m_hHandle = proc.hProcess;
	// Give the process time to execute and finish
	WaitForSingleObject(m_hHandle, INFINITE);
	m_ExecTime->Stop(&Elapsed);
	
	if (!GetExitCodeProcess(m_hHandle, &exitCode))      
	{
		m_StepStatus = gintFailed;
		LogSystemError(m_pErrorFile);
	}
	else
		m_StepStatus = gintComplete;
	
	// Close all open handles to the shelled process
	CloseHandle(m_hHandle);

	return((TC_TIME)Elapsed.int64);
}

STDMETHODIMP CExecute::AbortShell()
{
	if (m_hHandle != SQL_NULL_HSTMT)
		if (!TerminateProcess(m_hHandle, 0))
			return(RaiseSystemError());

	return(S_OK);
}

TC_TIME CExecute::ExecuteODBC()
{
	TC_TIME				tElapsed = 0;
	HDBC				m_hdbc;
	SQLRETURN			rc;
	LPSTR				szCmd;
	CURRENCY			Elapsed;
	BOOL				bDoConnect = FALSE;
	
	// ODBC specific initialization
	m_hdbc = SQL_NULL_HDBC;
	
	try
	{
		// Allocate a new connection if we are creating a dynamic connection or if
		// the named connection doesn't exist
		InitializeConnection(&m_hdbc, &bDoConnect);
		
		// Ensure that the connection is valid.
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLDriverConnect.\n");
#endif
		
		if (bDoConnect)
		{
			// Allocate connection handle, open a connection and set connection attributes.
#ifdef _DEBUG
			_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLDriverConnect.\n");
#endif
			
			if (m_bAbort)
				return(tElapsed);
			
			// Connect to the server using the passed in connection string
			rc = SQLDriverConnect(m_hdbc, NULL, 
				(unsigned char *)(LPSTR)m_szExecDtls, SQL_NTS, 
				NULL, 0, NULL, SQL_DRIVER_NOPROMPT); 
			HandleODBCError(rc, SQL_HANDLE_DBC, m_hdbc, SMSQLDriverConnect);
		}
		
		ReConnectDeadConnection(&m_hdbc);
		
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLAllocHandle for hdbc.\n");
#endif
		
		if (!m_bAbort && (rc = SQLAllocHandle(SQL_HANDLE_STMT, m_hdbc, &m_hHandle)) != SQL_SUCCESS)
			HandleODBCError(rc, SQL_HANDLE_DBC, m_hdbc, SMSQLAllocHandle);
		
		// Set connection attributes if any have been modified from the default values
		if (m_lRowCount > 0)
		{
			char			szConnOptions[512];
			
			sprintf(szConnOptions, "SET ROWCOUNT %d ", m_lRowCount);
			SetConnectionOption(szConnOptions, &m_hdbc);
		}
		
		if (m_bQuotedIds)
			SetConnectionOption("SET QUOTED_IDENTIFIER ON ", &m_hdbc);
		
		if (!m_bAnsiNulls)
			SetConnectionOption("SET ANSI_NULL_DFLT_OFF ON ", &m_hdbc);
		
		if (!m_bAbort && m_lQueryTmout > 0)
		{
#ifdef _DEBUG
			_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLSetStmtAttr.\n");
#endif
			
			// Set the query timeout on the statement handle
			rc = SQLSetStmtAttr(m_hHandle, SQL_ATTR_QUERY_TIMEOUT, &m_lQueryTmout, 
				SQL_IS_UINTEGER);
			HandleODBCError(rc, SQL_HANDLE_STMT, m_hHandle, SMSQLSetStmtAttr);
		}
		
		if (m_bNoExecute)
			SetConnectionOption("SET NOEXEC ON ", &m_hdbc);
		else if (m_bParseOnly)
			SetConnectionOption("SET PARSEONLY ON ", &m_hdbc);
		else if (m_bShowQP)
			// Important to ensure that this is the last connection attributes being set - 
			// otherwise showplans are generated for all remaining SET statements
			SetConnectionOption("SET SHOWPLAN_TEXT ON ", &m_hdbc);
		else 
		{
			if (m_bNoCount)
				SetConnectionOption("SET NOCOUNT ON ", &m_hdbc);
			
			if (m_bStatsIO)
				SetConnectionOption("SET STATISTICS IO ON ", &m_hdbc);
			
			// Important to ensure that this is the last connection attributes being set - 
			// otherwise timing statistics are generated for all remaining SET statements
			if (m_bStatsTime)
				SetConnectionOption("SET STATISTICS TIME ON ", &m_hdbc);
		}
		
		m_szCmd = (LPSTR)m_szCommand;
		m_ExecTime->Start();
		
		while ((szCmd = NextCmdInBatch((LPSTR)m_szCommand)) != NULL && !m_bAbort)
		{
#ifdef _DEBUG
			_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLExecDirect.\n");
#endif
			
			// Execute the ODBC command
			rc = SQLExecDirect(m_hHandle, (unsigned char *)szCmd, SQL_NTS);
			HandleODBCError(rc, SQL_HANDLE_STMT, m_hHandle, SMSQLExecDirect);
			
			free(szCmd);
			
			// Call a procedure to log the results to the output file
			ProcessResultsets();

		}
		m_ExecTime->Stop(&Elapsed);
		
		ResetConnectionProperties(&m_hdbc);
	}
	catch(CODBCError *pErr)
	{
		m_StepStatus = gintFailed;
		delete pErr;
	}
	catch(CExecError *pErr)
	{
		m_StepStatus = gintFailed;
		pErr->LogErrors(this);
		delete pErr;
	}
	
	ODBCCleanup(&m_hdbc, &m_hHandle);
	
	if (m_StepStatus != gintFailed)
		m_StepStatus = gintComplete;
	
	return((DWORD)Elapsed.int64);
}

void CExecute::InitializeConnection(HDBC *phdbc, BOOL *pbDoConnect)
{
	SQLRETURN			rc;

	*pbDoConnect = TRUE;

	if (IsDynamicConnection())
	{
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLAllocHandle for m_hdbc.\n");
#endif
		
		rc = SQLAllocHandle(SQL_HANDLE_DBC, henv, phdbc);
		HandleODBCError(rc, SQL_HANDLE_ENV, henv, SMSQLAllocHandle);

		return;
	}

	EnterCriticalSection(&hConnections);
	// Returns the connection handle if the connection, m_szConnection, exists
	for (m_iConnectionIndex = iConnectionCount - 1; m_iConnectionIndex >= 0; m_iConnectionIndex--)
	{
		if (!strcmp( (p_Connections + m_iConnectionIndex)->szConnectionName, (LPSTR)m_szConnection))
		{
			if (!(p_Connections + m_iConnectionIndex)->bInUse)
			{
				*phdbc = (p_Connections + m_iConnectionIndex)->hdbc;
				(p_Connections + m_iConnectionIndex)->bInUse = TRUE;

				*pbDoConnect = FALSE;
				break;
			}
			else
			{
				LeaveCriticalSection(&hConnections);
						
				throw new CExecError(CExecError::SM_ERR_CONN_IN_USE);
			}
		}
	}

	if (m_iConnectionIndex < 0)
	{
		// Connection was not found. Allocate connection handle and add it to list of 
		// available connections.
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLAllocHandle for m_hdbc.\n");
#endif
		
		rc = SQLAllocHandle(SQL_HANDLE_DBC, henv, phdbc);
		HandleODBCError(rc, SQL_HANDLE_ENV, henv, SMSQLAllocHandle);
		
		m_iConnectionIndex = iConnectionCount++;
		
		p_Connections = (SM_Connection_Info *)realloc(p_Connections, iConnectionCount * sizeof(SM_Connection_Info));
		
		strcpy((p_Connections + m_iConnectionIndex)->szConnectionName, (LPSTR)m_szConnection);
		(p_Connections + m_iConnectionIndex)->hdbc = *phdbc;
		(p_Connections + m_iConnectionIndex)->bInUse = TRUE;
	}
		
	LeaveCriticalSection(&hConnections);

	return;
}

void CExecute::ReConnectDeadConnection(HDBC *phdbc)
{
	SQLRETURN		rc;
	SQLUINTEGER		uConnDead;
	
	// Connect to the server using the passed in connection string
	rc = SQLGetConnectAttr(*phdbc, SQL_ATTR_CONNECTION_DEAD, 
		&uConnDead, SQL_IS_UINTEGER, NULL); 
	HandleODBCError(rc, SQL_HANDLE_DBC, *phdbc, SMSQLDriverConnect);
	
	if (uConnDead == SQL_CD_TRUE) 
	{
		// Cleanup the old connection and re-connect.
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLDisconnect.\n");
#endif
		rc = SQLDisconnect(*phdbc);
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLFreeHandle for hdbc.\n");
#endif
		SQLFreeHandle(SQL_HANDLE_DBC, *phdbc);
		*phdbc = SQL_NULL_HDBC;
		
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLDriverConnect.\n");
#endif

		rc = SQLAllocHandle(SQL_HANDLE_DBC, henv, phdbc);
		HandleODBCError(rc, SQL_HANDLE_ENV, henv, SMSQLAllocHandle);

		// Connect to the server using the passed in connection string
		rc = SQLDriverConnect(*phdbc, NULL, 
			(unsigned char *)(LPSTR)m_szExecDtls, SQL_NTS, 
			NULL, 0, NULL, SQL_DRIVER_NOPROMPT); 
		HandleODBCError(rc, SQL_HANDLE_DBC, *phdbc, SMSQLDriverConnect);
	}
	
	return;
}

void CExecute::ResetConnectionUsage()
{
	if(m_iConnectionIndex >= 0 && m_iConnectionIndex < iConnectionCount)
	{
		EnterCriticalSection(&hConnections);
		(p_Connections + m_iConnectionIndex)->bInUse = FALSE;
		LeaveCriticalSection(&hConnections);
	}
	
	return;
}

void CExecute::ResetConnectionProperties(HDBC *p_hdbc)
{
	SQLRETURN			rc;
	
	// Reset connection attributes if any have been modified from the default values
	
	if (m_bNoExecute)
		SetConnectionOption("SET NOEXEC OFF ", p_hdbc);
	else if (m_bParseOnly)
		SetConnectionOption("SET PARSEONLY OFF ", p_hdbc);
	else if (m_bShowQP)
		// Reset connection attributes in reverse order
		SetConnectionOption("SET SHOWPLAN_TEXT OFF ", p_hdbc);
	else 
	{
		// Reset connection attributes in reverse order
		if (m_bStatsTime)
			SetConnectionOption("SET STATISTICS TIME OFF ", p_hdbc);
		
		if (m_bNoCount)
			SetConnectionOption("SET NOCOUNT OFF ", p_hdbc);
		
		if (m_bStatsIO)
			SetConnectionOption("SET STATISTICS IO OFF ", p_hdbc);
	}
	
	if (m_lRowCount > 0)
	{
		char			szConnOptions[512];
		
		sprintf(szConnOptions, "SET ROWCOUNT 0 ");
		SetConnectionOption(szConnOptions, p_hdbc);
	}
	
	if (m_bQuotedIds)
		SetConnectionOption("SET QUOTED_IDENTIFIER OFF ", p_hdbc);
	
	if (!m_bAnsiNulls)
		SetConnectionOption("SET ANSI_NULL_DFLT_OFF OFF ", p_hdbc);
	
	if (m_lQueryTmout > 0)
	{
		SQLUINTEGER		lQueryTmout = 0;
		
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLSetStmtAttr.\n");
#endif
		
		// Set the query timeout on the statement handle
		rc = SQLSetStmtAttr(m_hHandle, SQL_ATTR_QUERY_TIMEOUT, &lQueryTmout, 
			SQL_IS_UINTEGER);
		HandleODBCError(rc, SQL_HANDLE_STMT, m_hHandle, SMSQLSetStmtAttr);
	}
	
	return;
}

LPSTR CExecute::NextCmdInBatch(LPSTR szBatch)
{
	LPSTR	szCmd, szSeparator, szStart;
	char	szNext;

	szStart = m_szCmd;

	while ( (szSeparator = strstr(szStart, CMD_SEPARATOR)) != NULL)
	{
		szNext = *(szSeparator + strlen(CMD_SEPARATOR));
		if ( szNext == '\n' || szNext == '\r' || szNext == '\0')
			break;
		else
			szStart = szSeparator + strlen(CMD_SEPARATOR);
	}

	if (!szSeparator)
	{
		// No more GO's
		if (strlen(m_szCmd) > 0)
		{
			szCmd = (LPSTR)malloc(strlen(m_szCmd) + 1);
			strcpy(szCmd, m_szCmd);
			m_szCmd += strlen(m_szCmd);
		}
		else
			szCmd = NULL;
	}
	else if (szSeparator - m_szCmd > 0)
	{
		// Strip the succeeding newline
		szCmd = (LPSTR)malloc(szSeparator - m_szCmd);
		strncpy(szCmd, m_szCmd, szSeparator - m_szCmd - 1);
		*(szCmd + (szSeparator - m_szCmd - 1)) = '\0';
		m_szCmd += szSeparator - m_szCmd + strlen(CMD_SEPARATOR);
		if ( szNext == '\n' || szNext == '\r')
			m_szCmd += 1;
	}
	else
		szCmd = NULL;

	return(szCmd);
}

void CExecute::SetConnectionOption(LPSTR szConn, HDBC *pHdbc)
{
	// Executes the passed in connection options 'set' statement. Returns True if it succeeded
	char			szConnOptions[512];
	SQLRETURN		rc;
	
	sprintf(szConnOptions, szConn);

#ifdef _DEBUG
	_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLExecDirect for connection option.\n");
#endif
		
	if (m_bAbort)
		return;
		
	rc = SQLExecDirect(m_hHandle, (unsigned char *)szConnOptions, SQL_NTS);
	HandleODBCError(rc, SQL_HANDLE_STMT, m_hHandle, SMSQLExecDirect);

	return;
}

STDMETHODIMP CExecute::AbortODBC()
{
	m_bAbort = TRUE;
	
	try
	{
		if (m_hHandle != SQL_NULL_HSTMT)
		{
#ifdef _DEBUG
			_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLCancel.\n");
#endif
			
			SQLRETURN rc = SQLCancel(m_hHandle);
			HandleODBCError(rc, SQL_HANDLE_STMT, m_hHandle, SMSQLCancel);
		}
	}
	catch(CODBCError *pErr)
	{
		delete pErr;
	}
	
	return(S_OK);
}

void CExecute::ProcessResultsets()
{
    SQLSMALLINT		*CTypeArray, *CScaleArray;
    SQLINTEGER		*ColLenArray, *DispLenArray, *OffsetArray;
	SQLSMALLINT		iColNameLen, SQLType, iColNull, i, NumCols = 0;
	SQLINTEGER		iDispLen, iRowCount;
    SQLRETURN		rc; 
	char			szColName[MAX_DATA_LEN + 1];
    void			*DataPtr;
	SQLINTEGER		iLenOrInd = ALIGNBUF(sizeof(SQLINTEGER));
	SQLUINTEGER		iRowArraySize, iArrayElementSize;
//	SQLUINTEGER		NumRowsFetched;
//	SQLUSMALLINT	*RowStatusArray;
    
	
	if (!m_pOutputFile || m_bAbort)
		return;
	
	do
	{
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLNumResultCols.\n");
#endif
		
		// Determine the number of result set columns.  
		rc = SQLNumResultCols(m_hHandle, &NumCols);
		HandleODBCError(rc, SQL_HANDLE_STMT, m_hHandle, SMSQLNumResultCols);
		
		if (NumCols > 0)
		{
			
			// Allocate arrays to hold the C type, scale, column and display length of the data
			CTypeArray = (SQLSMALLINT *) malloc(NumCols * sizeof(SQLSMALLINT));
			CScaleArray = (SQLSMALLINT *) malloc(NumCols * sizeof(SQLSMALLINT));
			ColLenArray = (SQLINTEGER *) malloc(NumCols * sizeof(SQLINTEGER));
			DispLenArray = (SQLINTEGER *) malloc(NumCols * sizeof(SQLINTEGER));

			OffsetArray = (SQLINTEGER *) malloc(NumCols * sizeof(SQLINTEGER));
			OffsetArray[0] = 0;
			
			for (i = 0; i < NumCols && !m_bAbort; i++) 
			{
#ifdef _DEBUG
				_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLDescribeCol.\n");
#endif
		
				// Get the column description, include the SQL type
				// Determine the column's byte length. Calculate the offset in the buffer to the
				// data as the offset to the previous column, plus the byte length of the previous
				// column, plus the byte length of the previous column's length/indicator buffer.
				// Note that the byte length of the column and the length/indicator buffer are increased
				// so that, assuming they start on an alignment boundary, they will end on the byte
				// before the next alignment boundary. Although this might leave some holes in the 
				// buffer, it is a relatively inexpensive way to guarantee alignment.
				rc = SQLDescribeCol(m_hHandle, ((SQLUSMALLINT) i)+1, 
					(unsigned char *)szColName, sizeof(szColName), &iColNameLen,
					&SQLType, (unsigned long *)&ColLenArray[i], &CScaleArray[i], &iColNull); 
				HandleODBCError(rc, SQL_HANDLE_STMT, m_hHandle, SMSQLDescribeCol);
				
#ifdef _DEBUG
				_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLColAttribute.\n");
#endif
		
				if (m_bAbort)
					return;

				rc = SQLColAttribute(m_hHandle, ((SQLUSMALLINT) i)+1, SQL_DESC_DISPLAY_SIZE, NULL, 0, NULL, &iDispLen);
				HandleODBCError(rc, SQL_HANDLE_STMT, m_hHandle, SMSQLColAttribute);
				
				// GetDefaultCType contains a switch statement that returns the default C type 
				// for each SQL type.
				CTypeArray[i] = GetDefaultCType(SQLType);
				if ( (CTypeArray[i] == SQL_C_CHAR || CTypeArray[i] == SQL_C_BINARY) && ColLenArray[i] > MAX_DATA_LEN)
				{
					ColLenArray[i] = MAX_DATA_LEN;
					iDispLen = MAX_DATA_LEN;
				}
				
				DispLenArray[i] = max(iColNameLen, iDispLen);
				DispLenArray[i] = max(DispLenArray[i], sizeof(S_NULL));
				
				// Print the column names in the header
				PrintData(szColName, SQL_C_CHAR, DispLenArray[i], 0, m_pOutputFile);
				
				// Add a byte for the null-termination character
				ColLenArray[i] += 1;
				ColLenArray[i] = ALIGNBUF(ColLenArray[i]);

				// Calculate the offset in the buffer to the data as the offset to the previous column, 
				// plus the byte length of the previous column, plus the byte length of the previous 
				// column's length/indicator buffer.
				if (i)
					OffsetArray[i] = OffsetArray[i-1] + ColLenArray[i-1] + iLenOrInd;
			}
			m_pOutputFile->WriteLine(NULL);

			iArrayElementSize = OffsetArray[NumCols-1] + ColLenArray[NumCols-1] + iLenOrInd;
			iRowArraySize = 1;
			// Allocate the data buffer. The size of the buffer is equal to the offset to the data
			// buffer for the final column, plus the byte length of the data buffer and length/indicator
			// buffer for the last column.
			DataPtr = malloc(iRowArraySize * iArrayElementSize);

			// Specify the size of the structure with the SQL_ATTR_ROW_BIND_TYPE 
			// statement attribute. This also declares that row-wise binding will 
			// be used. Declare the rowset size with the SQL_ATTR_ROW_ARRAY_SIZE 
			// statement attribute. Set the SQL_ATTR_ROW_STATUS_PTR statement 
			// attribute to point to the row status array. Set the 
			// SQL_ATTR_ROWS_FETCHED_PTR statement attribute to point to 
			// NumRowsFetched.
			/*
			RowStatusArray = (SQLUSMALLINT *)malloc(iRowArraySize * sizeof(SQLUSMALLINT));
			
			SQLSetStmtAttr(m_hHandle, SQL_ATTR_ROW_BIND_TYPE, &iArrayElementSize, SQL_IS_UINTEGER);
			SQLSetStmtAttr(m_hHandle, SQL_ATTR_ROW_ARRAY_SIZE, &iRowArraySize, SQL_IS_UINTEGER);
			SQLSetStmtAttr(m_hHandle, SQL_ATTR_ROW_STATUS_PTR, RowStatusArray, SQL_IS_POINTER);
			SQLSetStmtAttr(m_hHandle, SQL_ATTR_ROWS_FETCHED_PTR, &NumRowsFetched, SQL_IS_POINTER);
			*/
			
			// For each column, bind the address in the buffer at the start of the memory allocated
			// for that column's data and the address at the start of the memory allocated for that
			// column's length/indicator buffer.
			for (i = 0; i < NumCols; i++) 
			{
				SQLBindCol(m_hHandle, i + 1, CTypeArray[i], (SQLPOINTER)((SQLCHAR *)DataPtr + OffsetArray[i]),
						ColLenArray[i], (SQLINTEGER *)((SQLCHAR *)DataPtr + OffsetArray[i] + ColLenArray[i]));
				HandleODBCError(rc, SQL_HANDLE_STMT, m_hHandle, SMSQLBindCol);
			
				// Underline each column name
				memset(szColName, '-', DispLenArray[i]);
				*(szColName + DispLenArray[i]) = '\0';
				PrintData(szColName, SQL_C_CHAR, DispLenArray[i], 0, m_pOutputFile);
			}
			m_pOutputFile->WriteLine(NULL);
			
#ifdef _DEBUG
			_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLFetch.\n");
#endif
			
			while (!m_bAbort && (rc = SQLFetch(m_hHandle)) != SQL_NO_DATA) 
			{
				HandleODBCError(rc, SQL_HANDLE_STMT, m_hHandle, SMSQLFetch);
			
				/*
				for (i = 0; i < NumRowsFetched; i++) 
				{
					if (RowStatusArray[i] == SQL_ROW_SUCCESS|| RowStatusArray[i] == SQL_ROW_SUCCESS_WITH_INFO) 
					{
					*/
						for (i = 0; i < NumCols; i++) 
						{
							// Retrieve and print each row. PrintData accepts a pointer to the data, its C type,
							// and its byte length/indicator. 
							if ( *((SQLINTEGER *)((SQLCHAR *)DataPtr + OffsetArray[i] + ColLenArray[i])) == SQL_NULL_DATA)
								PrintData(S_NULL, SQL_C_CHAR, DispLenArray[i], 0, m_pOutputFile);
							else
								PrintData((LPVOID)((SQLCHAR *)DataPtr + OffsetArray[i]), CTypeArray[i], 
										DispLenArray[i], CScaleArray[i], m_pOutputFile);
						}
						m_pOutputFile->WriteLine(NULL);
						/*
					}
				}
				*/
			}
			m_pOutputFile->WriteLine(NULL);

			/*
			free(RowStatusArray);
			*/

			free(DataPtr);
			
			free(CTypeArray);
			free(CScaleArray);
			free(ColLenArray);
			free(DispLenArray);
		}
		
		// Write io statistics, if applicable
		LogODBCErrors(rc, SQL_HANDLE_STMT, m_hHandle, SMSQLFetch);
		
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLRowCount.\n");
#endif

		if (m_bAbort)
			break;

		// action (insert, update, delete) query
		rc = SQLRowCount(m_hHandle, &iRowCount);
		HandleODBCError(rc, SQL_HANDLE_STMT, m_hHandle, SMSQLRowCount);
		
		if (!m_bNoCount && iRowCount != -1)
		{
			sprintf(szColName, "(%d row(s) affected)", iRowCount);
			_bstr_t temp(szColName);
			m_pOutputFile->WriteLine((BSTR)temp);
			m_pOutputFile->WriteLine(NULL);
		}
		
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLFreeStmt.\n");
#endif
		
		if (m_bAbort)
			break;

		SQLFreeStmt(m_hHandle, SQL_UNBIND);
		
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLMoreResults.\n");
#endif
		
		if (m_bAbort)
			break;

		// Process the next resultset. This function returns 'success with info' even
		// if there is no other resultset and there are statistics messages to be printed.
		// Hence the check for -1 rows before printing. 
		rc=SQLMoreResults(m_hHandle);
		if (rc != SQL_NO_DATA)
			HandleODBCError(rc, SQL_HANDLE_STMT, m_hHandle, SMSQLMoreResults);
		
    } while (rc != SQL_NO_DATA);
	
	return;
}

void CExecute::PrintData(void *vData, SQLSMALLINT CType, SQLINTEGER IndPtr, SQLSMALLINT iScale, ISMLog *pOutput)
{
	// PrintData accepts a pointer to the data, its C type,
	// and its byte length/indicator. It contains a switch statement that casts and prints
	// the data according to its type. 
	
	char		*s;
	char		fmt[MAXBUFLEN];
	int			j = 0;
	SQLINTEGER	iColLen = IndPtr + 1; 

	assert(iColLen);
	s = (LPSTR)malloc(iColLen + 1);
	
	if (s)
	{
		if (vData)
		{
			switch(CType)
			{
			case SQL_C_CHAR:
			case SQL_C_WCHAR:
			case SQL_C_TYPE_DATE:
			case SQL_C_TYPE_TIME:
			case SQL_C_TYPE_TIMESTAMP:
			case SQL_C_INTERVAL_YEAR:
			case SQL_C_INTERVAL_MONTH:
			case SQL_C_INTERVAL_YEAR_TO_MONTH:
			case SQL_C_INTERVAL_DAY:
			case SQL_C_INTERVAL_HOUR:
			case SQL_C_INTERVAL_MINUTE:
			case SQL_C_INTERVAL_SECOND:
			case SQL_C_INTERVAL_DAY_TO_HOUR:
			case SQL_C_INTERVAL_DAY_TO_MINUTE:
			case SQL_C_INTERVAL_DAY_TO_SECOND:
			case SQL_C_INTERVAL_HOUR_TO_MINUTE:
			case SQL_C_INTERVAL_HOUR_TO_SECOND:
			case SQL_C_INTERVAL_MINUTE_TO_SECOND:
			case SQL_C_BINARY:
				sprintf(fmt, "%%.%ds", iColLen);
				j = sprintf(s, fmt, (char *)vData);
				break;
				
			case SQL_C_SHORT:
				j = sprintf(s, "%d", *(short *)vData);
				break;
				
			case SQL_C_LONG:
				j = sprintf(s, "%ld", *(long *)vData);
				break;
				
			case SQL_C_UBIGINT:
				j = sprintf(s, "%I64d", *(__int64 *)vData);
				break;
				
			case SQL_C_FLOAT:
				sprintf(fmt, "%%.0%df", iScale);
				j = sprintf(s, fmt, *(float *)vData);
				break;
				
			case SQL_C_DOUBLE:
			case SQL_C_NUMERIC:
				sprintf(fmt, "%%.0%df", iScale);
				j = sprintf(s, fmt, *(double *)vData);
				break;
				
			default:
				j = sprintf(s, "%s", vData);
				break;
			}
		}

		// Strip off terminating null character and pad the string with blanks
		if (iColLen - j > 0)
			memset(s + j, ' ', iColLen - j);
		
		*(s + iColLen) = '\0';
		
		// Write the field to the output file
		_bstr_t temp(s);
		pOutput->WriteField((BSTR)temp);
		free(s);
	}

	return;
}

SQLSMALLINT CExecute::GetDefaultCType(SQLINTEGER SQLType)
{
	// GetDefaultCType returns the C type for the passed in SQL datatype.
	
    switch(SQLType)
    {
    case SQL_CHAR:
	case SQL_VARCHAR:
	case SQL_LONGVARCHAR:
	case SQL_WCHAR:
	case SQL_WVARCHAR:
	case SQL_WLONGVARCHAR:
            return(SQL_C_CHAR);
        
    case SQL_TINYINT:
			return(SQL_C_CHAR);
        
    case SQL_SMALLINT:
			return(SQL_C_SHORT);
        
    case SQL_INTEGER:
			return(SQL_C_LONG);
        
    case SQL_BIGINT:
			return(SQL_C_UBIGINT);
        
    case SQL_REAL:
			return(SQL_C_FLOAT);
        
    case SQL_FLOAT:
    case SQL_DOUBLE:
//	case SQL_DECIMAL:
			return(SQL_C_DOUBLE);
        
	case SQL_DECIMAL:
			return(SQL_C_CHAR);
        
	case SQL_BIT:
			return(SQL_C_CHAR);
        
    case SQL_BINARY:
	case SQL_VARBINARY:
	case SQL_LONGVARBINARY:
			return(SQL_C_CHAR);
//			return(SQL_C_BINARY);
        
    case SQL_TYPE_DATE:
			return(SQL_C_CHAR);
//			return(SQL_C_TYPE_DATE);
        
    case SQL_TYPE_TIME:
			return(SQL_C_CHAR);
//			return(SQL_C_TYPE_TIME);
        
    case SQL_TYPE_TIMESTAMP:
			return(SQL_C_CHAR);
//			return(SQL_C_TYPE_TIMESTAMP);

	case SQL_NUMERIC:
            return(SQL_C_FLOAT);

    case SQL_INTERVAL_YEAR:
			return(SQL_C_CHAR);
//			return(SQL_C_INTERVAL_YEAR);

    case SQL_INTERVAL_MONTH:
			return(SQL_C_CHAR);
//			return(SQL_C_INTERVAL_MONTH);

    case SQL_INTERVAL_YEAR_TO_MONTH:
			return(SQL_C_CHAR);
//			return(SQL_C_INTERVAL_YEAR_TO_MONTH);

    case SQL_INTERVAL_DAY:
			return(SQL_C_CHAR);
//			return(SQL_C_INTERVAL_DAY);

    case SQL_INTERVAL_HOUR:
			return(SQL_C_CHAR);
//			return(SQL_C_INTERVAL_HOUR);

    case SQL_INTERVAL_MINUTE:
			return(SQL_C_CHAR);
//			return(SQL_C_INTERVAL_MINUTE);

    case SQL_INTERVAL_SECOND:
			return(SQL_C_CHAR);
//			return(SQL_C_INTERVAL_SECOND);

    case SQL_INTERVAL_DAY_TO_HOUR:
			return(SQL_C_CHAR);
//			return(SQL_C_INTERVAL_DAY_TO_HOUR);

    case SQL_INTERVAL_DAY_TO_MINUTE:
			return(SQL_C_CHAR);
//			return(SQL_C_INTERVAL_DAY_TO_MINUTE);

    case SQL_INTERVAL_DAY_TO_SECOND:
			return(SQL_C_CHAR);
//			return(SQL_C_INTERVAL_DAY_TO_SECOND);

    case SQL_INTERVAL_HOUR_TO_MINUTE:
			return(SQL_C_CHAR);
//			return(SQL_C_INTERVAL_HOUR_TO_MINUTE);

    case SQL_INTERVAL_HOUR_TO_SECOND:
			return(SQL_C_CHAR);
//			return(SQL_C_INTERVAL_HOUR_TO_SECOND);

    case SQL_INTERVAL_MINUTE_TO_SECOND:
			return(SQL_C_CHAR);
//			return(SQL_C_INTERVAL_MINUTE_TO_SECOND);

	default:
		assert(TRUE);
		return(SQL_C_CHAR);
		break;
    }
}

/*
    FUNCTION: LogODBCErrors(SQLRETURN rc, SWORD fHandleType, SQLHANDLE handle)
    COMMENTS: Formats ODBC errors or warnings and logs them. Also initializes the 
			  completion status for the step to failure, if an ODBC error has occurred.
*/

void CExecute::LogODBCErrors(SQLRETURN nResult, SWORD fHandleType, SQLHANDLE handle, OdbcOperations FailedOp)
{
	// Messages returned by the server (e.g. Print statements) will be logged to the output file
	// ODBC warnings will be logged to the log file
	// All other ODBC errors will be logged to the error file.

	UCHAR		szErrState[SQL_SQLSTATE_SIZE+1];		// SQL Error State string
	UCHAR		szErrText[SQL_MAX_MESSAGE_LENGTH+1];    // SQL Error Text string
	char		szBuffer[SQL_SQLSTATE_SIZE+SQL_MAX_MESSAGE_LENGTH+MAXBUFLEN+1] = "";   
														// formatted Error text Buffer
	SWORD		wErrMsgLen;								// Error message length
	SQLINTEGER	dwErrCode;								// Native Error code
	SQLRETURN	nErrResult;								// Return Code from SQLGetDiagRec
	SWORD		sMsgNum = 1;							// Error sequence number
	_bstr_t		temp;

	if (IsErrorReturn(nResult))
	{
		sprintf(szBuffer, "ODBC Operation: '%s' returned error code: %d", 
			g_szOdbcOps[FailedOp], nResult);
		temp = szBuffer;
		m_pErrorFile->WriteLine((BSTR) temp);
		m_StepStatus = gintFailed;
	}
			
	if (handle == SQL_NULL_HSTMT)
		return;

#ifdef _DEBUG
	_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLGetDiagRec.\n");
#endif

	// call SQLGetDiagRec function with proper ODBC handles, repeatedly until
	// function returns SQL_NO_DATA. 
	while (!m_bAbort && (nErrResult = SQLGetDiagRec(fHandleType, handle, sMsgNum++,
		szErrState, &dwErrCode, szErrText, SQL_MAX_MESSAGE_LENGTH-1, &wErrMsgLen))
		!= SQL_NO_DATA) 
	{
		if (!SQL_SUCCEEDED(nErrResult))
			break;
		
		if (m_pOutputFile && IsServerMessage(dwErrCode, szErrText))
		{
			wsprintf(szBuffer, SM_SQLMSG_FORMAT, (LPSTR)szErrText);
			temp = szBuffer;
			m_pOutputFile->WriteLine((BSTR) temp);
		}
		else if (IsODBCWarning(szErrState) && dwErrCode != SM_SQL_ERR_CHANGED_DB && dwErrCode != SM_SQL_ERR_CHANGED_LANG)
		{
			// Suppress warnings - 'Changed database context to...' and 'Changed language setting to...'
			wsprintf(szBuffer, SM_SQLMSG_FORMAT, ParseOdbcMsgPrefixes((LPCSTR)szErrText));
			temp = szBuffer;
			m_pOutputFile->WriteLine((BSTR) temp);
		}
		else if (m_pErrorFile && !IsODBCWarning(szErrState))
		{
			wsprintf(szBuffer, SM_SQLERR_FORMAT, (LPSTR)szErrState, dwErrCode, (LPSTR)szErrText);
			temp = szBuffer;
			m_pErrorFile->WriteLine((BSTR) temp);
		}
	}
}

/*
    FUNCTION: LogErrors(SQLRETURN rc, SWORD fHandleType, SQLHANDLE handle)
    COMMENTS: Writes the error message to the error log
*/

void CExecError::LogErrors(CExecute *p)
{
	_bstr_t temp(m_szExecErrorDesc[m_iErrCode]);

	if (p->m_pErrorFile)
		p->m_pErrorFile->WriteLine((BSTR)temp);

	return;
}

/*
    FUNCTION: ODBCCleanup(HDBC *hdbc, HSTMT *hstmt)
    COMMENTS: Cleanup of all ODBC structures
*/

void CExecute::ODBCCleanup(HDBC *hdbc, HSTMT *hstmt)
{
	SQLRETURN	lReturn;

#ifdef _DEBUG
	_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing ODBCCleanup.\n");
#endif

	if (*hstmt != SQL_NULL_HSTMT)
	{
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLCloseCursor.\n");
#endif
		SQLCloseCursor(hstmt);
#ifdef _DEBUG
		_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLFreeHandle for hstmt.\n");
#endif
		SQLFreeHandle(SQL_HANDLE_STMT, hstmt);
		*hstmt = SQL_NULL_HSTMT;
	}

	// Cleanup connection if it is a dynamic connection
	if (IsDynamicConnection())
	{
		if (*hdbc != SQL_NULL_HDBC)
		{
#ifdef _DEBUG
			_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLDisconnect.\n");
#endif
			lReturn = SQLDisconnect(*hdbc);
#ifdef _DEBUG
			_CrtDbgReport(_CRT_WARN, NULL, 0, NULL, "Executing SQLFreeHandle for hdbc.\n");
#endif
			SQLFreeHandle(SQL_HANDLE_DBC, hdbc);
			*hdbc = SQL_NULL_HDBC;
		}
	}
	else
		ResetConnectionUsage();

	return;
}

// Wrapper function that raises an error if a Windows Api fails
STDMETHODIMP CExecute::RaiseSystemError(void)
{
	char s[MAXBUFLEN];
	
	GetSystemError(s);
	return Error(s, 0, NULL, GUID_NULL);
}

// Wrapper function that logs the error raised by an Api function to the passed in file
void CExecute::LogSystemError(ISMLog *pFile)
{
	if (pFile)
	{
		char s[MAXBUFLEN];
		GetSystemError(s);
		
		_bstr_t temp(s);
		pFile->WriteLine((BSTR)temp);
	}
}

// Populates the passed in string with the last Windows Api error that occurred
void CExecute::GetSystemError(LPSTR s)
{
	long c;
	DWORD e;
	
	e = GetLastError();
	
	c = sprintf(s, "Error code: %ld. ", e);
	c = FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM | FORMAT_MESSAGE_IGNORE_INSERTS, 
		NULL, e, 0, s + c, MAXBUFLEN - c, NULL);
	
	return;
}

STDMETHODIMP CExecute::get_StepStatus(InstanceStatus *pVal)
{
	*pVal = m_StepStatus;
	return S_OK;
}

STDMETHODIMP CExecute::WriteError(BSTR szMsg)
{
	if (m_pErrorFile)
		return(m_pErrorFile->WriteLine(szMsg));

	return S_OK;
}

