Attribute VB_Name = "ConnectionCommon"
'  FILE:       ConnnectionCommon.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains functionality common across StepMaster and
'              SMRunOnly, pertaining to connection strings
'              Specifically, functions to load connections strings
'              in an array and so on.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "ConnectionCommon."

Public Sub LoadRecordsetInConnectionArray(rstConns As Recordset, cConns As cConnections)

    Dim cNewConnection As cConnection
    
    On Error GoTo LoadRecordsetInConnectionArrayErr

    If rstConns.RecordCount = 0 Then
        Exit Sub
    End If
   
    rstConns.MoveFirst
    While Not rstConns.EOF
    
        Set cNewConnection = New cConnection
        
        ' Initialize Connection values
        ' Call a procedure to raise an error if mandatory fields are null.
        cNewConnection.ConnectionId = ErrorOnNullField(rstConns, "connection_id")
        cNewConnection.WorkspaceId = CStr(ErrorOnNullField(rstConns, FLD_ID_WORKSPACE))
        cNewConnection.ConnectionName = CStr(ErrorOnNullField(rstConns, "connection_name"))
        cNewConnection.ConnectionValue = CheckForNullField(rstConns, "connection_value")
        cNewConnection.Description = CheckForNullField(rstConns, "description")

        cNewConnection.NoCountDisplay = CheckForNullField(rstConns, "no_count_display")
        cNewConnection.NoExecute = CheckForNullField(rstConns, "no_execute")
        cNewConnection.ParseQueryOnly = CheckForNullField(rstConns, "parse_query_only")
        cNewConnection.QuotedIdentifiers = CheckForNullField(rstConns, "ANSI_quoted_identifiers")
        cNewConnection.AnsiNulls = CheckForNullField(rstConns, "ANSI_nulls")
        cNewConnection.ShowQueryPlan = CheckForNullField(rstConns, "show_query_plan")
        cNewConnection.ShowStatsTime = CheckForNullField(rstConns, "show_stats_time")
        cNewConnection.ShowStatsIO = CheckForNullField(rstConns, "show_stats_io")
        cNewConnection.ParseOdbcMsg = CheckForNullField(rstConns, "parse_odbc_msg_prefixes")
        cNewConnection.RowCount = CheckForNullField(rstConns, "row_count")
        cNewConnection.TsqlBatchSeparator = CheckForNullField(rstConns, "tsql_batch_separator")
        cNewConnection.QueryTimeOut = CheckForNullField(rstConns, "query_time_out")
        cNewConnection.ServerLanguage = CheckForNullField(rstConns, "server_language")
        cNewConnection.CharacterTranslation = CheckForNullField(rstConns, "character_translation")
        cNewConnection.RegionalSettings = CheckForNullField(rstConns, "regional_settings")
        
        cConns.Load cNewConnection
        
        Set cNewConnection = Nothing
        rstConns.MoveNext
    Wend
    
    Exit Sub
    
LoadRecordsetInConnectionArrayErr:
    LogErrors Errors
    gstrSource = mstrModuleName & "LoadRecordsetInConnectionArray"
    On Error GoTo 0
    Err.Raise vbObjectError + errLoadRsInArrayFailed, gstrSource, _
            LoadResString(errLoadRsInArrayFailed)
End Sub
