Attribute VB_Name = "TreeSM"
'  FILE:       TreeSM.bas
'              Microsoft TPC-H Kit Ver. 2.7.0-1005
'              Copyright Microsoft, 2008
'              All Rights Reserved
'
'
'  PURPOSE:    Contains functions to manipulate the tree view
'              in the left-hand pane of the explorer-style frmMain.
'              Specifically, it contains functions to add nodes
'              to the tree view, expand a node and so on.
'  Contact:    Reshma Tharamal (reshmat@microsoft.com)
'
Option Explicit

' Used to indicate the source module name when errors
' are raised by this module
Private Const mstrModuleName As String = "TreeSM."
Public Sub ExpandNode(treExplore As comctllib.TreeView, _
        lvwExplore As comctllib.ListView, _
        Optional ByVal bExpand As Boolean = True)

    ' This procedure is called when the user selects the ExpandAll or CollapseAll
    ' options on the context-sensitive menu. Expands/collapses all the nodes beneath the
    ' currently selected node
    
    Dim strKey As String
    Dim nodToExpand As Node
    Dim cExpTree As cNavTree
    
    On Error GoTo ExpandNodeErr
    
    strKey = ItemSelectedInTree(treExplore)
    
    If Not StringEmpty(strKey) Then
    
        ' The node that has been selected by the user should
        ' be expanded
        Set nodToExpand = treExplore.Nodes(strKey)
        
        ' Call a procedure to expand the node data in the
        ' list view
        Call ExploreNode(treExplore, lvwExplore, nodToExpand)
        
        ' And explode the tree
        Set cExpTree = New cNavTree
        
        Set cExpTree.Tree = treExplore
        cExpTree.RootNode = strKey
        cExpTree.Direction = gintTopDown
                
        Do While (Not StringEmpty(strKey))
            treExplore.Nodes(strKey).Expanded = bExpand
            strKey = cExpTree.NextNode
        Loop
        Set cExpTree = Nothing
    End If
    
    Exit Sub
    
ExpandNodeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    gstrSource = mstrModuleName & "ExpandNode"
    Call ShowError(errExploreFailed)
    
End Sub
Public Function AddNodeToTree(treStepView As TreeView, _
    strKey As String, strStringToAdd As String, _
    lngWorkspaceId As Long, intNodeType As Integer, _
    objNode As Object, _
    Optional strParentNodeKey As String, _
    Optional ByVal SiblingKey As String = gstrEmptyString) As comctllib.Node
    
    ' Adds a node to the tree view control and returns
    ' the node that has been added
    
    Dim nodTreeNode As Node
    Dim strParentKey As String
    Dim intImageIndex As Integer
    
    On Error GoTo AddNodeToTreeErr
    
    ' Retrieve the index of the image for the node being added
    ' to the tree
    intImageIndex = GetCollapsedIcon(intNodeType, objNode)
    
    Select Case intNodeType
        Case gintWorkspace
            Set nodTreeNode = treStepView.Nodes.Add(, , _
                    strKey, strStringToAdd, intImageIndex)
        
        Case gintParameter
            strParentKey = LabelParameter(lngWorkspaceId)
            Set nodTreeNode = treStepView.Nodes.Add( _
                    strParentKey, tvwChild, _
                    strKey, strStringToAdd, intImageIndex)
            
        Case gintNodeParamBuiltIn
            strParentKey = LabelBuiltIns(lngWorkspaceId)
            Set nodTreeNode = treStepView.Nodes.Add( _
                    strParentKey, tvwChild, _
                    strKey, strStringToAdd, intImageIndex)
            
        Case gintNodeParamConnection
            strParentKey = LabelParamConnection(lngWorkspaceId)
            Set nodTreeNode = treStepView.Nodes.Add( _
                    strParentKey, tvwChild, _
                    strKey, strStringToAdd, intImageIndex)
            
        Case gintConnectionDtl
            strParentKey = LabelConnectionDtl(lngWorkspaceId)
            Set nodTreeNode = treStepView.Nodes.Add( _
                    strParentKey, tvwChild, _
                    strKey, strStringToAdd, intImageIndex)
            
        Case gintNodeParamExtension
            strParentKey = LabelAppExtension(lngWorkspaceId)
            Set nodTreeNode = treStepView.Nodes.Add( _
                    strParentKey, tvwChild, _
                    strKey, strStringToAdd, intImageIndex)
            
        Case gintGlobalStep
            If StringEmpty(strParentNodeKey) Then
                strParentKey = LabelGlobalStep(lngWorkspaceId)
            Else
                strParentKey = strParentNodeKey
            End If
            Set nodTreeNode = treStepView.Nodes.Add( _
                    strParentKey, tvwChild, _
                    strKey, strStringToAdd, intImageIndex)
                
        Case gintManagerStep, gintWorkerStep
            If StringEmpty(SiblingKey) Then
                If StringEmpty(strParentNodeKey) Then
                    strParentKey = LabelStep(lngWorkspaceId)
                Else
                    strParentKey = strParentNodeKey
                End If
                Set nodTreeNode = treStepView.Nodes.Add( _
                        strParentKey, tvwChild, _
                        strKey, strStringToAdd, intImageIndex)
            Else
                Set nodTreeNode = treStepView.Nodes.Add( _
                        SiblingKey, tvwPrevious, _
                        strKey, strStringToAdd, intImageIndex)
            End If
                
        Case gintGlobalsLabel
        Case Else
            'Do nothing
    End Select
    
    ' Call a procedure to get the icon of the expanded node
    intImageIndex = GetExpandedIcon(intNodeType, objNode)
    nodTreeNode.ExpandedImage = intImageIndex
    
    Set AddNodeToTree = nodTreeNode
    
    Exit Function
    
AddNodeToTreeErr:
    ' Log the error code raised by Visual Basic
    Call LogErrors(Errors)
    On Error GoTo 0
    gstrSource = mstrModuleName & "AddNodeToTree"
    Err.Raise vbObjectError + errAddNodeToTreeFailed, _
            gstrSource, _
            LoadResString(errAddNodeToTreeFailed)

End Function
Private Function ParameterNode(intNodeType As Integer) As Boolean

    ParameterNode = ((intNodeType = gintParameter) Or _
            (intNodeType = gintNodeParamBuiltIn) Or _
            (intNodeType = gintNodeParamExtension))

End Function
Private Function ConnectionNode(intNodeType As Integer) As Boolean

    ConnectionNode = (intNodeType = gintNodeParamConnection)

End Function
Private Function ConnectionDtl(intNodeType As Integer) As Boolean

    ConnectionDtl = (intNodeType = gintConnectionDtl)

End Function

Public Sub UpdateTreeNode(treStepView As TreeView, strNodeKey As String, _
        strNodeText As String, objNodeData As Object, intTypeOfNode As Integer)
        
    On Error GoTo UpdateTreeNodeErr
    
    treStepView.Nodes(strNodeKey).Text = strNodeText
    If IsStep(intTypeOfNode) Then
        treStepView.Nodes(strNodeKey).ExpandedImage = GetExpandedIcon(intTypeOfNode, _
                bEnabled:=objNodeData.EnabledFlag)
        treStepView.Nodes(strNodeKey).Image = GetCollapsedIcon(intTypeOfNode, _
                bEnabled:=objNodeData.EnabledFlag)
    End If
    
    Exit Sub
    
UpdateTreeNodeErr:
    Call LogErrors(Errors)
    BugMessage "Update Tree Node errored : Resuming..."
    ' Since this is not a critical error, log the error and resume
    Resume Next
        
End Sub
Public Sub RemoveSubTree(treTreeView As TreeView, strRootNode As String, lvwSteps As ListView)

    ' Removes all the children of the root node from the passed
    ' in control. Finally removes the root node.
    
    Dim nodCurrent As Node
    Dim nodNext As Node
    Dim lngIdentifier As Long
    Dim intNodeType As Integer
    Dim lngWorkspace As Long
    
    On Error GoTo RemoveSubTreeErr
    
    lngWorkspace = WorkspaceSelected(strRootNode)
    
    If treTreeView.Nodes(strRootNode).Children > 0 Then
        Set nodNext = treTreeView.Nodes(strRootNode).Child
    End If
    
    ' While more step nodes exist
    ' The loop is necessary in order to remove all steps and parameters
    ' from the corresponding arrays, if any exist in the workspace
    While treTreeView.Nodes(strRootNode).Children > 0
        Set nodCurrent = nodNext
        
        If nodCurrent.Children > 0 Then
            Set nodNext = nodCurrent.Child
        Else
            
            If nodCurrent.Index <> nodCurrent.LastSibling.Index Then
                ' Set the next node pointer to the next sibling if it
                ' exists
                Set nodNext = nodCurrent.Next
            ElseIf nodCurrent.Parent.Key <> strRootNode Then
                ' Set the next node pointer to the parent.
                ' Once we get to the root node, we are done
                Set nodNext = nodCurrent.Parent
            End If
        
            lngIdentifier = MakeIdentifierValid(nodCurrent.Key)
            intNodeType = TypeOfNode(nodCurrent.Key)
            
            ' Since the current node does not have any children,
            ' remove the node from the corresponding collection
            If Not IsLabel(nodCurrent.Key) Then
                If ParameterNode(intNodeType) Then
                    gcParameters.Unload lngIdentifier
                ElseIf ConnectionNode(intNodeType) Then
                    gcConnections.Unload lngIdentifier
                ElseIf ConnectionDtl(intNodeType) Then
                    gcConnDtls.Unload lngIdentifier
                ElseIf StepNode(nodCurrent.Key) Then
                    gcSteps.Unload lngIdentifier
                    
                    ' Unload all constraints for the step
                    gcConstraints.UnloadStepConstraints lngIdentifier
                        
                Else
                    On Error GoTo 0
                    Err.Raise vbObjectError + errInvalidNode, _
                            gstrSource, _
                            LoadResString(errInvalidNode)
                End If
            End If
            
            ' Since the node does not have any children, we
            ' can remove it
            Call DeleteViewNode(treTreeView, lvwSteps, lngIdentifier, _
                    intNodeType, lngWorkspace)
        End If
    Wend
    
    ' Remove the root node from the tree and list views
    lngIdentifier = MakeIdentifierValid(strRootNode)
    intNodeType = TypeOfNode(strRootNode)
    Call DeleteViewNode(treTreeView, lvwSteps, lngIdentifier, _
            intNodeType, lngWorkspace)
    
    Exit Sub

RemoveSubTreeErr:
    Call LogErrors(Errors)
    On Error GoTo 0
    Err.Raise vbObjectError + errUnableToRemoveSubtree, _
            gstrSource, _
            LoadResString(errUnableToRemoveSubtree)

End Sub
